<?php
/**
 * Plugin Name: MerchBridge Personalized
 * Plugin URI: https://merchbridge.com/
 * Description: A plugin for WooCommerce, display personalized fields.
 * Version: 0.1.0
 * Author: MerchBridge
 * Author URI: https://merchbridge.com/about
 * Text Domain: mb-personalized
 */

defined( 'ABSPATH' ) || exit;
define( 'MB_PERSONALIZED_DIR', plugin_dir_path( __FILE__ ) );
define( 'MB_PERSONALIZED_URL', plugin_dir_url( __FILE__ ) );

if ( ! class_exists( 'Woo_Variation_Swatches' ) ) {
	require_once MB_PERSONALIZED_DIR . 'vendor/woo-variation-swatches/woo-variation-swatches.php';
}

class MB_Personalized {
	public function __construct() {
		add_action( 'woocommerce_before_add_to_cart_button', array( $this, 'render_personalize_fields' ) );
		add_filter( 'woocommerce_add_cart_item_data', array( $this, 'save_add_to_cart_meta_data' ), 10, 4 );
		add_action( 'woocommerce_get_item_data', array( $this, 'push_add_to_cart_meta_data' ), 10, 2 );
		add_action( 'woocommerce_checkout_create_order_line_item', array( $this, 'save_order_item_meta_data' ), 10, 4 );
		add_filter( 'woocommerce_display_item_meta', array( $this, 'checkout_page_display_item_meta' ), 10, 3 );
		add_action( 'wp_enqueue_scripts', array( $this, 'enqueue_scripts' ), 1000 );

		add_filter( 'manage_edit-product_columns', array( $this, 'admin_products_personalized_column' ) );
		add_action( 'manage_product_posts_custom_column', array( $this, 'admin_products_personalized_column_content' ), 10, 2 );

		add_action(
			'woocommerce_after_order_itemmeta',
			function( $item_id, $item, $product ) {
				if ( is_admin() && is_object( $product ) && method_exists( $product, 'get_id' ) ) {
					$item_meta_data = $item->get_meta_data();
					if ( is_array( $item_meta_data ) && ! empty( $item_meta_data ) ) {
						foreach ( $item_meta_data as $item_meta ) {
							if ( is_object( $item_meta ) && method_exists( $item_meta, 'get_data' ) ) {
								$data = $item_meta->get_data();
								if ( is_array( $data ) && ! empty( $data ) ) {
									if ( isset( $data['key'] ) && '_mb_personalized' == $data['key'] && isset( $data['value'] ) && is_array( $data['value'] ) && ! empty( $data['value'] ) ) {
										echo '<div><p><strong>Customization:</strong></p></div>';
										echo '<div style="margin-left: 10px;">';
										foreach ( $data['value'] as $da ) {
											if ( isset( $da['title'] ) && ! empty( $da['title'] ) && isset( $da['value'] ) ) {
												echo sprintf( '<p><strong>+ %s:</strong> %s</p>', $da['title'], $da['value'] );
											}
										}
										echo '</div>';
									}
								}
							}
						}
					}
					
				}

			},
			9999999,
			3
		);
	}

	function admin_products_personalized_column( $columns ) {
		if ( ! isset( $columns['is_personalized'] ) ) {
			$columns['is_personalized'] = 'Personalized';
		}
		return $columns;
	}

	function admin_products_personalized_column_content( $column, $product_id ) {
		if ( 'is_personalized' == $column ) {
			$personalized_settings = get_post_meta( $product_id, '_mb_personalized_configs', true );
			if ( ! empty( $personalized_settings ) ) {
				$decode_settings = json_decode( $personalized_settings, true );

				if ( is_array( $decode_settings ) && ! empty( $decode_settings ) ) {
					echo '<span style="color: #fff;background-color: #00d97e;display: inline-block;padding: 4px 8px;font-size: 12px;font-weight: 400;line-height: 1;text-align: center;white-space: nowrap;vertical-align: baseline;border-radius: 5px;">Personalized</span>';
				}
			}
		}
	}

	public function enqueue_scripts() {
		if ( function_exists( 'is_product' ) && is_product() ) {
			wp_enqueue_style( 'chosen-js', MB_PERSONALIZED_URL . 'assets/css/chosen.min.css' );
			wp_enqueue_style( 'mb-personalized-style', MB_PERSONALIZED_URL . 'assets/css/frontend.css' );

			wp_enqueue_script( 'chosen-js', MB_PERSONALIZED_URL . 'assets/js/chosen.jquery.min.js', array( 'jquery' ) );
			wp_enqueue_script( 'mb-personalized-js', MB_PERSONALIZED_URL . 'assets/js/frontend.js', array( 'jquery' ) );
		}

	}

	public function render_personalize_fields() {
		global $product;
		if ( is_object( $product ) && method_exists( $product, 'get_id' ) ) {
			$product_id = $product->get_id();
			if ( is_numeric( $product_id ) && $product_id > 0 ) {
				$personalized_settings = get_post_meta( $product_id, '_mb_personalized_configs', true );
				if ( ! empty( $personalized_settings ) ) {
					$decode_settings = json_decode( $personalized_settings, true );

					if ( is_array( $decode_settings ) && ! empty( $decode_settings ) ) {
						?>
						<div class="mb-personlized-wrap">
						<?php
						foreach ( $decode_settings as $key => $value ) {
							$field_type = ( isset( $value['field_type'] ) ) ? $value['field_type'] : '';
							$valid_type = array(
								'text',
								'number',
								'textarea',
								'checkbox',
								'radio',
								'dropdown',
							);
							if ( in_array( $field_type, $valid_type ) ) {
								$placeholder = ( isset( $value['configs'] ) && isset( $value['configs']['placeholder'] ) ) ? $value['configs']['placeholder'] : '';
								$description = ( isset( $value['configs'] ) && isset( $value['configs']['description'] ) ) ? $value['configs']['description'] : '';

								$field_type    = ( isset( $value['field_type'] ) ) ? $value['field_type'] : '';
								$default_value = ( isset( $value['configs'] ) && isset( $value['configs']['settings'] ) && isset( $value['configs']['settings']['defaultValue'] ) ) ? $value['configs']['settings']['defaultValue'] : '';
								if ( in_array( $field_type, array( 'checkbox', 'dropdown', 'radio' ) ) ) {
									$options = ( isset( $value['configs'] ) && isset( $value['configs']['settings'] ) && isset( $value['configs']['settings']['options'] ) ) ? $value['configs']['settings']['options'] : '';
								}
								$visibility = ( isset( $value['configs'] ) && isset( $value['configs']['visibility'] ) && isset( $value['configs']['visibility']['enabled'] ) && $value['configs']['visibility']['enabled'] ) ? $value['configs']['visibility'] : array();

								$data_field_name = sprintf( ' data-field_name="%s"', esc_attr( $value['field_name'] ) );

								$condition_attr = '';
								if ( is_array( $visibility ) && isset( $visibility['rules'] ) && ! empty( $visibility['rules'] ) ) {
									$condition_attr .= 'data-enable_condition="yes"';

									if ( isset( $visibility['match'] ) ) {
										$condition_attr .= sprintf( ' data-condition_match="%s"', $visibility['match'] );
									}
									if ( isset( $visibility['type'] ) ) {
										$condition_attr .= sprintf( ' data-condition_type="%s"', $visibility['type'] );
									}
									$condition_attr .= " data-condition_rules='" . json_encode( $visibility['rules'] ) . "'";
								}
								$attr_required     = ( isset( $value['configs'] ) && isset( $value['configs']['required'] ) && $value['configs']['required'] ) ? 'required="required"' : '';
								$class_required    = ( isset( $value['configs'] ) && isset( $value['configs']['required'] ) && $value['configs']['required'] ) ? 'mb-required' : '';
								$is_field_required = ( isset( $value['configs'] ) && isset( $value['configs']['required'] ) && $value['configs']['required'] ) ? true : false;

								$template_dir = MB_PERSONALIZED_DIR . "inc/templates/field-type/{$field_type}.php";
								if ( file_exists( $template_dir ) ) {
									include $template_dir;
								}
							}
						}
						?>
						</div>
						<?php
					}
				}
			}
		}
	}


	public function save_add_to_cart_meta_data( $cart_item_data, $product_id, $variation_id, $quantity ) {
		if ( isset( $_REQUEST['pm_personalized'] ) ) {
			$cart_item_data['pm_personalized'] = $_REQUEST['pm_personalized'];
		}
		return $cart_item_data;
	}

	public function push_add_to_cart_meta_data( $item_data, $cart_item_data ) {
		if ( isset( $cart_item_data['pm_personalized'] ) && ! empty( $cart_item_data['pm_personalized'] ) ) {
			$data = $cart_item_data['pm_personalized'];
			foreach ( $data as $key => $value ) {
				if ( isset( $value['title'] ) && ! empty( $value['title'] ) && isset( $value['value'] ) && ! empty( $value['value'] ) ) {
					$data_title    = $value['title'];
					$data_value    = $value['value'];
					$display_value = $data_value;
					if ( is_array( $data_value ) ) {
						$display_value = implode( ', ', $data_value );
					}
					$item_data[ $key ] = array(
						'key'     => $data_title,
						'value'   => $data_value,
						'display' => $display_value,
					);
				}
			}
		}
		return $item_data;
	}

	public function save_order_item_meta_data( $item, $item_data, $values, $order ) {
		if ( is_array( $values ) && isset( $values['pm_personalized'] ) && ! empty( $values['pm_personalized'] ) ) {
			$personalized = $values['pm_personalized'];
			$item->add_meta_data( '_mb_personalized', $personalized, true );
		}
	}

	public function checkout_page_display_item_meta( $html, $item, $args ) {
		$data_personalized = $item->get_meta( '_mb_personalized' );
		if ( is_array( $data_personalized ) && ! empty( $data_personalized ) ) {
			$strings = array();
			foreach ( $data_personalized as $key => $value ) {
				if ( isset( $value['title'] ) && ! empty( $value['title'] ) && isset( $value['value'] ) && ! empty( $value['value'] ) ) {
					$data_title    = $value['title'];
					$data_value    = $value['value'];
					$display_value = $data_value;
					if ( is_array( $data_value ) ) {
						$display_value = implode( ', ', $data_value );
					}
					$value     = $args['autop'] ? wp_kses_post( $display_value ) : wp_kses_post( trim( $display_value ) );
					$strings[] = $args['label_before'] . wp_kses_post( $data_title ) . $args['label_after'] . $value;
				}
			}
			if ( $strings ) {
				$html .= $args['before'] . implode( $args['separator'], $strings ) . $args['after'];
			}
		}
		return $html;
	}
}

new MB_Personalized();
